class Group(object):
    def __init__(self, json):
        self.uuid = json['uuid']
        self.id = json['links'][0]['id']
        self.name = json['name']
        self.href = json['links'][0]['href']

class VaultRecord(object):
    def __init__(self, json):
        self.uuid = json['uuid']
        self.name = json['name']
        self.color = json['color'] if 'color' in json else ''
        self.username = json['username'] if 'username' in json else ''
        self.url = json['url'] if 'url' in json else ''
        self.additionalObjects = json['additionalObjects']
        self.password = json['additionalObjects']['secret']['password'] if 'secret' in json['additionalObjects'] and 'password' in json['additionalObjects']['secret'] else ''
        self.totp = json['additionalObjects']['secret']['totp'] if 'secret' in json['additionalObjects'] and 'totp' in json['additionalObjects']['secret'] else ''
        self.filename = json['filename'] if 'filename' in json else ''
        self.file = json['additionalObjects']['secret']['file'] if 'secret' in json['additionalObjects'] and 'file' in json['additionalObjects']['secret'] else ''


class KeyHubVault(object):
    def __init__(self, authentication):
        self._session = authentication.session
        self._uri = authentication.uri

    def get_group(self, group_uuid):
        response = self._session.get(self._uri + "/keyhub/rest/v1/group?uuid=" + group_uuid)
        return Group(response.json()['items'][0])
    
    def get_vault_records(self, group_uuid):
        # TODO: support for client vault -> group_uuid=None
        group = self.get_group(group_uuid)
        response = self._session.get(group.href + '/vault')
        records = []
        for record in response.json()['records']:
            records.append(VaultRecord(record))
        return records

    def get_vault_record(self, group_uuid, record_uuid):
        group = self.get_group(group_uuid)
        response = self._session.get(group.href + "/vault/record/uuid/" + record_uuid + "?additional=secret&groupid=" + group.id) 
        return VaultRecord(response.json())
        
    def post_vault_record(self, group_uuid, payload):
        group = self.get_group(group_uuid)
        response = self._session.post(group.href + "/vault/record?additional=secret", json=payload)
        return VaultRecord(response.json()['items'][0])

def vault(authentication):
    return KeyHubVault(authentication=authentication)




class Account(object):
    def __init__(self, json):
        self.href = json['links'][0]['href']
        self.validity = json['validity']
        self.uuid = json['uuid']
        self.username = json['username']
        self.displayName = json['displayName']
        self.lastActive = json['lastActive']
        self.links = json['links']
        self.active = json['active']
        self.reregistrationRequired = json['reregistrationRequired']
        self.validInDirectory = json['validInDirectory']
        self.canRequestGroups = json['canRequestGroups']
        self.tokenPasswordEnabled = json['tokenPasswordEnabled']
        self.keyHubPasswordChangeRequired = json['keyHubPasswordChangeRequired']
        self.directoryPasswordChangeRequired = json['directoryPasswordChangeRequired']
        self.licenseRole = json['licenseRole']


class KeyHubAccount(object):
    def __init__(self, authentication):
        self._session = authentication.session
        self._uri = authentication.uri
    
    def get_account_records(self, account_uuid = False, account_username = False):
        if account_uuid:
            response = self._session.get(self._uri + "/keyhub/rest/v1/account?uuid=" + account_uuid)
        elif account_username:
            response = self._session.get(self._uri + "/keyhub/rest/v1/account?username=" + account_username)
        else:
            raise Exception('one of the following variables must be defined: account_uuid or account_username')
        
        records = []
        for record in response.json()['items']:
            records.append(Account(record))
        return records

    def get_accounts_in_state(self, active):
        response = self._session.get(self._uri + "/keyhub/rest/v1/account?active=" + str(active))
        records = []
        for record in response.json()['items']:
            records.append(Account(record))
        return records


def account(authentication):
    return KeyHubAccount(authentication=authentication)



from oauthlib.oauth2 import BackendApplicationClient
from requests_oauthlib import OAuth2Session
import requests, json

class KeyHubClientAuthentication(object):
    def __init__(self, uri, client_id, client_secret):
        self.uri = uri
        self.client_id = client_id
        self.client_secret = client_secret

        client = BackendApplicationClient(client_id=client_id)
        oauth = OAuth2Session(client=client)
        token = oauth.fetch_token(token_url=uri+'/login/oauth2/token?authVault=access', client_id=client_id, client_secret=client_secret)

        session = OAuth2Session(client_id, token=token)
        session.headers.update({'Accept': 'application/vnd.topicus.keyhub+json;version=latest'})
        session.headers.update({'topicus-Vault-session': token['vaultSession']})
        self.session = session
    
    def info(self):
        # TODO: return info object instead of json
        response = self.session.get(self.uri + "/keyhub/rest/v1/info")
        return response.json()

    def docs(self):
        response = requests.get(self.uri + "/keyhub/rest/v1/openapi.json")
        return response.json()

class KeyHubAccountAuthentication(object):
    def __init__(self, uri, client_id, client_secret, callback_uri):
        # https://keyhub-instance/login/code?usercode=CODE-HERE
        # "authorizationCode" : {
        #   "authorizationUrl" : "https://keyhub-instance/login/oauth2/authorize",
        #    "tokenUrl" : "https://keyhub-instance/login/oauth2/token",
        #    "refreshUrl" : "https://keyhub-instance/login/oauth2/token",
        self.uri = uri
        authorize_url = self.uri + "/login/oauth2/authorize"
        token_url = self.uri + "/login/oauth2/token"
        
        authorization_redirect_url = authorize_url + '?response_type=code&client_id=' + client_id + '&redirect_uri=' + callback_uri + '&scope=openid'

        print("go to the following url on the browser and enter the code from the returned url: ")
        print("---  " + authorization_redirect_url + "  ---")
        authorization_code = input('code: ')

        # step I, J - turn the authorization code into a access token, etc
        data = {'grant_type': 'authorization_code', 'code': authorization_code, 'redirect_uri': callback_uri}
        print("requesting access token")
        access_token_response = requests.post(token_url, data=data, verify=True, allow_redirects=False, auth=(client_id, client_secret))
        
        # we can now use the access_token as much as we want to access protected resources.
        tokens = json.loads(access_token_response.text)
        access_token = tokens['access_token']
        headers = {'Authorization': 'Bearer ' + access_token}
        self.headers = headers
        

def client_auth(uri, client_id, client_secret):
    return KeyHubClientAuthentication(uri=uri, client_id=client_id, client_secret=client_secret)

def account_auth(uri, client_id, client_secret, callback_uri):
    return KeyHubAccountAuthentication(uri=uri, client_id=client_id, client_secret=client_secret, callback_uri=callback_uri)



from keyhub.authentication import client_auth, account_auth
from keyhub.account import account
from keyhub.vault import vault
from keyhub.profile import profile
from keyhub.misc import misc
from keyhub.client import client

__all__ = (
    'client_auth',
    'account_auth',
    'account',
    'vault',
    'misc',
    'profile',
    'client',
)



import requests

class Client(object):
    def __init__(self, json):
        self.href = json['links'][0]['href']
        self.uuid = json['uuid']
        self.clientId = json['clientId']
        self.type = json['type']
        self.name = json['name']
        self.ssoApplication = json['ssoApplication']
        self.confidential = json['confidential']
        self.showLandingPage = json['showLandingPage']
        self.useClientCredentials = json['useClientCredentials']


class KeyHubClient(object):
    def __init__(self, authentication):
        self._session = authentication.session
        self._uri = authentication.uri
        self._client_id = authentication.client_id
        self._client_secret = authentication.client_secret
        self._session_password = authentication.session.access_token

    def get_client_info(self):
        response = self._session.get(self._uri + "/keyhub/rest/v1/client/me")
        return Client(response.json())

    def post_client_vault(self):
        content = { '$type': 'vault.VaultUnlock', 'password': self._session_password }
        response = requests.post(self._uri + "/keyhub/rest/v1/client/vault/unlock", data=content, verify=True, allow_redirects=False, auth=(self._client_id, self._client_secret))
        return response

def client(authentication):
    return KeyHubClient(authentication=authentication)



import requests

class Profile(object):
    def __init__(self, json):
        self.href = json['links'][0]['href']
        self.validity = json['validity']
        self.uuid = json['uuid']
        self.username = json['username']
        self.displayName = json['displayName']
        self.lastActive = json['lastActive']
        self.active = json['active']
        self.reregistrationRequired = json['reregistrationRequired']
        self.validInDirectory = json['validInDirectory']
        self.canRequestGroups = json['canRequestGroups']
        self.tokenPasswordEnabled = json['tokenPasswordEnabled']
        self.keyHubPasswordChangeRequired = json['keyHubPasswordChangeRequired']
        self.directoryPasswordChangeRequired = json['directoryPasswordChangeRequired']
        self.licenseRole = json['licenseRole']


class KeyHubProfile(object):
    def __init__(self, authentication):
        self._headers = authentication.headers
        self._uri = authentication.uri
    
    def get_profile_info(self):
        # response = self._session.get(self._uri + "/keyhub/rest/v1/account/me") 
        response = requests.get(self._uri + "/keyhub/rest/v1/account/me", headers=self._headers, verify=True)
        return response.text

    def get_rotating_password(self):
        # /keyhub/rest/v1/account/provisioning/tokenpwd
        response = requests.get(self._uri + "/keyhub/rest/v1/account/provisioning/tokenpwd", headers=self._headers, verify=True)
        # return Profile(response.json())
        return response.text


def profile(authentication):
    return KeyHubProfile(authentication=authentication)




# Untested

import requests

class Group(object):
    def __init__(self, json):
        self.auditedGroups = json['auditedGroups']
        self.unauditedGroups = json['unauditedGroups']
        self.overdueAuditGroups = json['overdueAuditGroups']
        self.validVaultPolicies = json['validVaultPolicies']
        self.vaultsWithoutPolicies = json['vaultsWithoutPolicies']
        self.overdueVaultPolicies = json['overdueVaultPolicies']

        self.groupClassificationsUuid = json['classifications'][0]['uuid']
        self.groupClassificationsName = json['classifications'][0]['name']
        self.groupClassificationsDefaultClassification = json['classifications'][0]['defaultClassification']
        self.groupClassificationsMaxAuditInterval = json['classifications'][0]['maximumAuditInterval']
        self.groupClassificationsAuditMonths = json['classifications'][0]['requiredMonths']
        self.groupClassificationsAuditTrail = json['classifications'][0]['recordTrailRequired']
        self.groupClassificationsRotPasswd = json['classifications'][0]['rotatingPasswordRequired']
        self.groupClassificationsAuthGroupProv = json['classifications'][0]['authorizingGroupProvisioningRequired']
        self.groupClassificationsAuthGroupMembershp = json['classifications'][0]['authorizingGroupMembershipRequired']
        self.groupClassificationsAuthGroupGroupAudit = json['classifications'][0]['authorizingGroupAuditingRequired']
        self.groupClassificationsVaultAct = json['classifications'][0]['vaultRequiresActivation']
        self.groupClassificationsMinNrManagers = json['classifications'][0]['minimumNrManagers']
        self.groupClassificationsDesc = json['classifications'][0]['description']


class Misc(object):
    def __init__(self, authentication):
        self._headers = authentication.headers
        self._uri = authentication.uri

    def get_auditstats(self):
        response = requests.get(self._uri + "/keyhub/rest/v1/group/auditstats", headers=self._headers, verify=True)
        # return Group(response.json())
        return response.text

    def get_certificates(self):
        response = requests.get(self._uri + "/keyhub/rest/v1/certificate", headers=self._headers, verify=True)
        # return Group(response.json())
        return response.text

def misc(authentication):
    return Misc(authentication=authentication)




# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup

setup(
    name='keyhub',
    version='0.2.0',
    description='Python client for the Topicus KeyHub Vault API',
    keywords='keyhub',
    url='https://www.topicus-keyhub.com/',
    author='Marc Mast',
    author_email='connect.with.marck@gmail.com',
    license='Apache Software License 2.0',
    packages=['keyhub'],
    install_requires=[
        'requests_oauthlib>=0.8.0',
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
    zip_safe=False
)
